@@ -1,4 +1,8 @@
 import itertools as it
+from util import transitive_get as walk
+from util import deep_transitive_get as walkstar
+from util import (assoc, unique, unique_dict, interleave, take, evalt, isempty,
+        intersection)
 
 class Var(object):
     """ Logic Variable """
@@ -8,14 +12,13 @@ class Var(object):
             token = wild()
         elif len(token) == 1:
             token = token[0]
-        # return (Var, token)
 
         obj = object.__new__(cls)
         obj.token = token
         return obj
 
     def __str__(self):
-        return str(self.token)
+        return "~" + str(self.token)
     __repr__ = __str__
 
     def __eq__(self, other):
@@ -26,45 +29,10 @@ class Var(object):
 
 var = lambda *args: Var(*args)
 isvar = lambda t: isinstance(t, Var)
-# isvar = lambda t: isinstance(t, tuple) and len(t) >= 1 and t[0] is Var
 
 class wild(object):
     pass
 
-def transitive_get(key, d):
-    """ Transitive dict.get
-
-    >>> d = {1: 2, 2: 3, 3: 4}
-    >>> d.get(1)
-    2
-    >>> transitive_get(1, d)
-    4
-    """
-    while key in d:
-        key = d[key]
-    return key
-
-def deep_transitive_get(key, d):
-    """ Transitive get that propagates within tuples
-
-    >>> d = {1: (2, 3), 2: 12, 3: 13}
-    >>> transitive_get(1, d)
-    (2, 3)
-    >>> deep_transitive_get(1, d)
-    (12, 13)
-    """
-
-    key = transitive_get(key, d)
-    if isvar(key):
-        return key
-    elif isinstance(key, tuple):
-        return tuple(map(lambda k: deep_transitive_get(k, d), key))
-    else:
-        return key
-
-walk = transitive_get
-walkstar = deep_transitive_get
-
 def reify(e, s):
     if isvar(e):
         return walkstar(e, s)
@@ -73,11 +41,6 @@ def reify(e, s):
     else:
         return e
 
-def assoc(dict, key, value):
-    d = dict.copy()
-    d[key] = value
-    return d
-
 def unify(u, v, s):  # no check at the moment
     """ Find substitution so that u == v while satisfying s
 
@@ -86,7 +49,7 @@ def unify(u, v, s):  # no check at the moment
     """
     u = walk(u, s)
     v = walk(v, s)
-    if u is v:
+    if u == v:
         return s
     if isvar(u):
         return assoc(s, u, v)
@@ -102,23 +65,6 @@ def unify(u, v, s):  # no check at the moment
         return s
     return False
 
-def unique_dict(seq):
-    seen = set()
-    for d in seq:
-        h = hash(frozenset(d.items()))
-        if h not in seen:
-            seen.add(h)
-            yield d
-
-def unique(seq):
-    seen = set()
-    for item in seq:
-        if item not in seen:
-            seen.add(item)
-            yield item
-
-# TODO: replace chain with interleave
-
 def conde(*goalseqs):
     """ Logical cond
 
@@ -131,15 +77,6 @@ def conde(*goalseqs):
                                      for goals in goalseqs))
     return goal_conde
 
-def interleave(seqs):
-    iters = map(iter, seqs)
-    while iters:
-        for itr in iters:
-            try:
-                yield next(itr)
-            except StopIteration:
-                iters.remove(itr)
-
 def bind(stream, goal):
     """ Bind a goal to a stream
 
@@ -180,13 +117,6 @@ def run(n, x, *goals):
     """
     return take(n, unique(walkstar(x, s) for s in bindstar(({},), *goals)))
 
-def take(n, seq):
-    if n is None:
-        return seq
-    if n == 0:
-        return tuple(seq)
-    return tuple(it.islice(seq, 0, n))
-
 
 # Goals
 def fail(s):
@@ -243,48 +173,6 @@ def goal_tuple_eval(goalt):
     """
     return lambda s: evalt(reify(goalt, s))(s)
 
-def evalt(t):
-    """ Evaluate tuple if unevaluated
-
-    >>> add = lambda x, y: x + y
-    >>> evalt((add, 2, 3))
-    5
-    >>> evalt(add(2, 3))
-    5
-    """
-
-    if isinstance(t, tuple) and len(t) >= 1 and callable(t[0]):
-        return t[0](*t[1:])
-    else:
-        return t
-
-def isempty(it):
-    """ Is an iterable empty
-
-    destructive.  Use with tee
-
-    >>> from itertools import tee
-    >>> it = range(3)
-    >>> tmp, it = tee(it, 2)
-    >>> isempty(tmp)
-    False
-    """
-    try:
-        next(iter(it))
-        return False
-    except StopIteration:
-        return True
-
-def intersection(*seqs):
-    for item in seqs[0]:
-        found = True
-        for seq in seqs[1:]:
-            if item not in seq:
-                found = False
-                break
-        if found:
-            yield item
-
 class Relation(object):
     def __init__(self):
         self.facts = set()
@@ -328,6 +216,7 @@ def fact(rel, *args):
     ('Homer',)
     """
     rel.add_fact(*args)
+
 def facts(rel, *lists):
     """ Declare several facts
 
@@ -352,21 +241,21 @@ def facts(rel, *lists):
 """
 
 def heado(x, coll):
-    if coll:
+    if isinstance(coll, tuple) and len(coll) >= 1:
         return eq(x, coll[0])
     else:
         return fail
 
 def tailo(x, coll):
-    if coll:
+    if isinstance(coll, tuple) and len(coll) >= 1:
         return eq(x, coll[1:])
     else:
         return fail
 
-def appendo(l, s, out):
+def appendo(l, s, ls):
     """ Byrd thesis pg. 247 """
     a, d, res = [var(wild()) for i in range(3)]
-    return conde((eq(l, ()), eq(s, out)),
+    return conde((eq(l, ()), eq(s, ls)),
                  ((heado, a, l),   (tailo, d, l),
-                  (heado, a, out), (tailo, res, out),
+                  (heado, a, ls), (tailo, res, ls),
                   (appendo, d, s, res)))
